%
% SOLUTION AND IMPLEMENTATION OF DISTRIBUTED LIFESPAN MODELS
%
% Gilbert Koch(*) and Johannes Schropp
%
% (*) Department of Pharmaceutical Sciences, State University of
%     New York at Buffalo, 403 Kapoor Hall, Buffalo, New York 14214, USA
%     Tel.: +1-716-6454816
%     Fax: +1-716-8296569
%     E-mail: gilbertk@buffalo.edu
%
%
% Version: September 30, 2013
%
%


clear all;  
clear global all;
warning off;

global g_t_data g_y_data gv_start_vals g_param;
global gs_distr_N g_mean_N g_var_N g_a_N g_b_N; 
global g_n g_t;
global g_dose g_doses;
global g_anzobj g_anz_per_obj;
global g_Tol_QUAD g_Tol_ODE;

options = odeset('RelTol',1e-8,'AbsTol',1e-8);
opt_MLE = optimset('MaxFunEvals',2500);

% Setting for lsqcurvefit
lsq_jac = 'off';         % Explicit gradient            
diff_quot = 'forward';   % 'forward' or 'central'
lsq_ops = [1e-8, 1e-8, 1e5, 1e5];   % TolFun, TolX, MaxIter,MaxFunEvals
options_lsq = optimset('Jacobian',lsq_jac,'TolFun',lsq_ops(1),...
                       'TolX',lsq_ops(2),'MaxIter',lsq_ops(3),...
                       'MaxFunEvals',lsq_ops(4),...
                       'Algorithm','levenberg-marquardt',...
                       'FinDiffType',diff_quot);
                   
% Setting for fminsearch                   
simplex_ops = [1e-8, 1e-8, 500, 1e5];                   
opt_simplex = optimset('TolFun',simplex_ops(1),...
                       'TolX',simplex_ops(2),'MaxIter',simplex_ops(3),...
                       'MaxFunEvals',simplex_ops(4));
                   
randn('state',0);       % Reset RNG

marker = {'black x','black s','black +','black o'};
       
g_anzobj = 4;

% Produce artificial data for several doses
% ========================================================================

gs_distr_N = 'weibull';    
% gs_distr_N = 'lognormal';
% gs_distr_N = 'gamma';

s_perturbation = 'off';     % off medium strong

g_mean_N = 5;
g_var_N = 1;
switch gs_distr_N
    case 'weibull'
        [g_a_N g_b_N] = calc_moment2param_weibull(g_mean_N,g_var_N);
    case 'lognormal'
        [g_a_N g_b_N] = calc_moment2param_lognorm(g_mean_N,g_var_N);
    case 'gamma'
        [g_a_N g_b_N] = calc_moment2param_gamma(g_mean_N,g_var_N);
end
        
disp(sprintf('Create artificial data based on integral representation'));
disp(sprintf('gs_distr_N = %s  s_perturbation = %s',gs_distr_N,s_perturbation));
disp(sprintf('g_mean_N = %12.6f  g_var_N = %12.6f',g_mean_N,g_var_N));
disp(sprintf('g_a_N    = %12.6f  g_b_N   = %12.6f',g_a_N,g_b_N));

g_param = [0.5 10 1 g_a_N g_b_N];

g_a = g_a_N;
g_b = g_b_N;


% Create data
% ------------------------------------------------------------------------

g_doses = [10 50 100 500];
g_anz_per_obj = 26; 
g_Tol_QUAD = 1e-8;

% Plot kin
% g_dose = g_doses(1);
% v_t = [-5:0.1:20];
% v_kin = fun_kin(g_param(1),g_param(2),g_param(3),v_t);
% figure(2); plot(v_t,v_kin,'black','LineWidth',2); hold on;
% xlabel('Time t','FontSize',14);
% ylabel('k_{in}(t)','FontSize',14);

for d = 1:g_anzobj
    g_dose = g_doses(d);

    for i=1:2000
        t = i/100;
        t_plot(i) = t;
        pk_plot(i) = fun_c_PKpo_1Komp(t);       % PK given explicitly
        % pk_plot(i) = fun_c_PKpo_1Komp_ODE(t);   % PK given via an ODE
    end
    figure(1); semilogy(t_plot,pk_plot,'black','LineWidth',2); title('PK','FontSize',14); hold on;
    xlabel('Time t','FontSize',14); ylabel('Drug concentration c(t)','FontSize',14);
    % figure(2); plot(t_plot,kin_plot,'black'); title('I(c)'); hold on;

    cavg(d) = trapz(t_plot,pk_plot) / 20;
    
    % Calc DLSM
    clear t_plot N_plot N_plot_per
    for i=1:g_anz_per_obj
        switch s_perturbation
            case 'off'
                err = 0; 
            case 'medium'
                err = randn/5;
            case 'strong' 
                err = randn/2;  
        end
        
        t = i-1; 
        
        t_plot(i) = t;
        N_plot(i) = calc_DLSM_EXPL(t);
        N_plot_per(i) = N_plot(i)+err;  
        
        g_t_data((d-1)*g_anz_per_obj+i) = t_plot(i);
        g_y_data((d-1)*g_anz_per_obj+i) = N_plot_per(i);
    end

    t_save = t_plot;
    y_save(d,:) = N_plot_per;
    
    figure(5); plot(t_plot,N_plot,'black','LineWidth',2); hold on;
    figure(5); plot(t_plot,N_plot_per,marker{d},'MarkerSize',8,'LineWidth',2); hold on;
    figure(10); plot(t_plot,N_plot_per,marker{d},'MarkerSize',8,'LineWidth',2); hold on;
end

% void = create_adapt_data('data_adapt.dat',t_save,y_save);
% void = create_nonmem_data('data_nonmem.txt',t_save,y_save);


g_dose = g_doses(1);

for i=1:500
    c = i;
    c_plot(i) = c;
    I_plot(i) = (g_param(1)*c)/(g_param(2)+c);
end

% Plot distribution
void = plot_distributions_N(gs_distr_N,g_param(4),g_param(5),'black');


disp('Taste druecken');
pause;


% Fitting
%==========================================================================

s_model = 'EXPL_TRAPEZ';          % Integral solution with trapez and LB
% s_model = 'EXPL_TRAPEZ_SIMPLEX';  % Integral solution with trapez and Simplex
% s_model = 'EXPL_QUAD';            % Integral solution with QUAD and LB
% s_model = 'EXPL_QUAD_SIMPLEX';    % Integral solution with QUAD and Simplex
% s_model = 'EXPL_ODE';             % Integral solution with ODE45 and LB
% s_model = 'EXPL_ODE_SIMPLEX';     % Integral solution with ODE45 and Simplex

% s_model = 'ROC';                  % Rate of change with LB
% s_model = 'ROC_SIMPLEX';          % Rate of change with Simplex

% s_model = 'APPROX_KRZYZ';         % Krzyzanski with LB
% s_model = 'APPROX_KRZYZ_SIMPLEX'; % Krzyzanski with Simplex


g_mean_N = 5;
g_var_N = 1;
switch gs_distr_N
    case 'weibull'
        [g_a_N g_b_N] = calc_moment2param_weibull(g_mean_N,g_var_N);
    case 'lognormal'
        [g_a_N g_b_N] = calc_moment2param_lognorm(g_mean_N,g_var_N);
    case 'gamma'
        [g_a_N g_b_N] = calc_moment2param_gamma(g_mean_N,g_var_N);
end



lb = []; ub = [];   

[a b] = calc_moment2param(gs_distr_N,6,3);
gv_start_vals = [0.7 5 0.8 a b];   
   

switch s_model
                    
    case {'EXPL_TRAPEZ','EXPL_TRAPEZ_SIMPLEX'} 
         g_n = 100;
         
    case {'EXPL_QUAD','EXPL_QUAD_SIMPLEX'}
         g_Tol_ODE = 1e-8;
         
    case {'EXPL_ODE','EXPL_ODE_SIMPLEX'}
         g_Tol_ODE = 1e-8;
         
    case {'ROC','ROC_SIMPLEX'}
        g_Tol_QUAD = 1e-8;
        g_Tol_ODE = 1e-8; 
                                         
    case {'APPROX_KRZYZ','APPROX_KRZYZ_SIMPLEX'}                
        g_n = 50;  
        g_Tol_ODE = 1e-7; 
                          
end


disp('Fitting');
tic_fit = tic;

switch s_model
            
    case 'EXPL_TRAPEZ'
        [erg,resnorm,residual,exitflag,output,lambda,jacobian]  = ... 
            lsqcurvefit(@fun_F_DLSM_EXPL_TRAPEZ,gv_start_vals,...
                        g_t_data,g_y_data,lb,ub,options_lsq);
                            
    case 'EXPL_TRAPEZ_SIMPLEX'
        [erg,fval,exitflag,output] = ... 
            fminsearch(@fun_SofS_DLSM_EXPL_TRAPEZ,gv_start_vals,opt_simplex);
           
    case 'EXPL_QUAD'
        [erg,resnorm,residual,exitflag,output,lambda,jacobian]  = ... 
            lsqcurvefit(@fun_F_DLSM_EXPL_QUAD,gv_start_vals,...
                        g_t_data,g_y_data,lb,ub,options_lsq); 
                        
    case 'EXPL_QUAD_SIMPLEX'
        [erg,fval,exitflag,output] = ... 
            fminsearch(@fun_SofS_DLSM_EXPL_QUAD,gv_start_vals,opt_simplex);
           
    case 'EXPL_ODE'
        [erg,resnorm,residual,exitflag,output,lambda,jacobian]  = ... 
            lsqcurvefit(@fun_F_DLSM_EXPL_ODE,gv_start_vals,...
                        g_t_data,g_y_data,lb,ub,options_lsq); 
                    
    case 'EXPL_ODE_SIMPLEX'
        [erg,fval,exitflag,output] = ... 
            fminsearch(@fun_SofS_DLSM_EXPL_ODE,gv_start_vals,opt_simplex);
                        
    case 'ROC'
        [erg,resnorm,residual,exitflag,output,lambda,jacobian]  = ... 
            lsqcurvefit(@fun_F_DLSM_ROC,gv_start_vals,...
                        g_t_data,g_y_data,lb,ub,options_lsq); 
                            
     case 'ROC_SIMPLEX'
        [erg,fval,exitflag,output] = ... 
            fminsearch(@fun_SofS_DLSM_ROC,gv_start_vals,opt_simplex);
                          
     case 'APPROX_KRZYZ'
        [erg,resnorm,residual,exitflag,output]  = ... 
            lsqcurvefit(@fun_F_DLSM_KRZYZ,gv_start_vals,...
                        g_t_data,g_y_data,lb,ub,options_lsq); 
                            
     case 'APPROX_KRZYZ_SIMPLEX'
      [erg,fval,exitflag,output] = ... 
            fminsearch(@fun_SofS_DLSM_KRZYZ_SIMPLEX,gv_start_vals,opt_simplex); 
                            

    
end    
time_fit = toc(tic_fit)
Iterations = output.iterations

g_param = erg;
  

% Plot Results
%=========================================================================

g_param = erg;

switch gs_distr_N
    case 'weibull'
        [mean var] = calc_param2moment_weibull(g_param(4),g_param(5));  
    case 'lognormal'
        [mean var] = calc_param2moment_lognorm(g_param(4),g_param(5));
    case 'gamma'
        [mean var] = calc_param2moment_gamma(g_param(4),g_param(5));
end

t_plot = [0:0.25:25];

for d=1:4
    g_dose = g_doses(d);

    switch s_model
    
        case {'EXPL_TRAPEZ','EXPL_TRAPEZ_SIMPLEX'}
            delta = mean/g_n;
            nalpha = calc_nalpha(g_n,g_param(4),g_param(5),0.01,delta,gs_distr_N,g_n*10);
            for k=1:length(t_plot)
                g_t = t_plot(k);
                y_plot(k) = calc_Trapez_vect(@fun_f,delta,nalpha);
            end

        case {'EXPL_QUAD','EXPL_QUAD_SIMPLEX'}
            for k=1:length(t_plot)
                y_plot(k) = calc_DLSM_EXPL(t_plot(k));
            end
            
        case {'EXPL_ODE','EXPL_ODE_SIMPLEX'}
            for k=1:length(t_plot)
                y_plot(k) = calc_DLSM_EXPL_ODE(t_plot(k));
            end
            
        case {'ROC','ROC_SIMPLEX'}
            x0 = mean*g_param(3);
            sol = ode45(@fun_model_DLSM_ROC,[0,25+1],x0,options);
            for k=1:length(t_plot)
                y_plot(k) = deval(sol,t_plot(k));
            end 
                    
        case {'APPROX_KRZYZ','APRROX_KRZYZ_SIMPLEX'}
            dtau = (5*2)/g_n;
            for i=1:g_n+1
                x0(i) = g_param(3)*(i-1)*dtau;
            end
            x0(g_n+2) = mean*g_param(3);
            sol = ode45(@fun_model_DLSM_KRZYZ,[0,25+1],x0,options);
            for k=1:length(t_plot)
                Y = deval(sol,t_plot(k));
                y_plot(k) = Y(g_n+2);
            end
    end
    figure(10); plot(t_plot,y_plot,'red','LineWidth',2); hold on;
end
          
        
      





